@@ -1,24 +1,21 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
-# License: http://pysnmp.sf.net/license.html
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
 #
 import socket
 from pysnmp.carrier.base import AbstractTransportAddress
 from pysnmp.carrier.asyncio.dgram.base import DgramAsyncioProtocol
-try:
-    import asyncio
-except ImportError:
-    import trollius as asyncio
 
-loop = asyncio.get_event_loop()
 
 domainName = snmpUDP6Domain = (1, 3, 6, 1, 2, 1, 100, 1, 2)
 
+
 class Udp6TransportAddress(tuple, AbstractTransportAddress):
     pass
 
+
 class Udp6AsyncioTransport(DgramAsyncioProtocol):
     sockFamily = socket.has_ipv6 and socket.AF_INET6 or None
     addressType = Udp6TransportAddress
@@ -28,9 +25,10 @@ class Udp6AsyncioTransport(DgramAsyncioProtocol):
             return self.addressType((transportAddress[0].split('%')[0],
                                      transportAddress[1],
                                      0,  # flowinfo
-                                     0)) # scopeid
+                                     0))  # scopeid
         else:
             return self.addressType((transportAddress[0],
                                      transportAddress[1], 0, 0))
 
+
 Udp6Transport = Udp6AsyncioTransport
