@@ -1,4 +1,9 @@
 #
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
+#
 # Copyright (C) 2014, Zebra Technologies
 # Authors: Matt Hooks <me@matthooks.com>
 #          Zachary Lorusso <zlorusso@gmail.com>
@@ -26,6 +31,8 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 # THE POSSIBILITY OF SUCH DAMAGE.
 #
+import sys
+
 from pysnmp.smi.rfc1902 import *
 from pysnmp.hlapi.auth import *
 from pysnmp.hlapi.context import *
@@ -33,8 +40,10 @@ from pysnmp.hlapi.lcd import *
 from pysnmp.hlapi.varbinds import *
 from pysnmp.hlapi.asyncio.transport import *
 from pysnmp.entity.rfc3413 import cmdgen
+
 try:
     import asyncio
+
 except ImportError:
     import trollius as asyncio
 
@@ -45,12 +54,13 @@ lcd = CommandGeneratorLcdConfigurator()
 
 isEndOfMib = lambda x: not cmdgen.getNextVarBinds(x)[1]
 
+
 @asyncio.coroutine
 def getCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Creates a generator to perform SNMP GET query.
 
-    When itereator gets advanced by :py:mod:`asyncio` main loop,
+    When iterator gets advanced by :py:mod:`asyncio` main loop,
     SNMP GET request is send (:RFC:`1905#section-4.2.1`).
     The iterator yields :py:class:`asyncio.Future` which gets done whenever
     response arrives or error occurs.
@@ -120,15 +130,23 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBinds, cbCtx):
         lookupMib, future = cbCtx
         if future.cancelled():
             return
-        future.set_result(
-            (errorIndication, errorStatus, errorIndex, vbProcessor.unmakeVarBinds(snmpEngine, varBinds, lookupMib))
-        )
+        try:
+            varBindsUnmade = vbProcessor.unmakeVarBinds(snmpEngine, varBinds,
+                                                        lookupMib)
+        except Exception:
+            ex = sys.exc_info()[1]
+            future.set_exception(ex)
+        else:
+            future.set_result(
+                (errorIndication, errorStatus, errorIndex, varBindsUnmade)
+            )
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
 
@@ -142,12 +160,13 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
     )
     return future
 
+
 @asyncio.coroutine
 def setCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Creates a generator to perform SNMP SET query.
 
-    When itereator gets advanced by :py:mod:`asyncio` main loop,
+    When iterator gets advanced by :py:mod:`asyncio` main loop,
     SNMP SET request is send (:RFC:`1905#section-4.2.5`).
     The iterator yields :py:class:`asyncio.Future` which gets done whenever
     response arrives or error occurs.
@@ -217,15 +236,23 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBinds, cbCtx):
         lookupMib, future = cbCtx
         if future.cancelled():
             return
-        future.set_result(
-            (errorIndication, errorStatus, errorIndex, vbProcessor.unmakeVarBinds(snmpEngine, varBinds, lookupMib))
-        )
+        try:
+            varBindsUnmade = vbProcessor.unmakeVarBinds(snmpEngine, varBinds,
+                                                        lookupMib)
+        except Exception:
+            ex = sys.exc_info()[1]
+            future.set_exception(ex)
+        else:
+            future.set_result(
+                (errorIndication, errorStatus, errorIndex, varBindsUnmade)
+            )
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
 
@@ -239,12 +266,13 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
     )
     return future
 
+
 @asyncio.coroutine
 def nextCmd(snmpEngine, authData, transportTarget, contextData,
             *varBinds, **options):
     """Creates a generator to perform SNMP GETNEXT query.
 
-    When itereator gets advanced by :py:mod:`asyncio` main loop,
+    When iterator gets advanced by :py:mod:`asyncio` main loop,
     SNMP GETNEXT request is send (:RFC:`1905#section-4.2.2`).
     The iterator yields :py:class:`asyncio.Future` which gets done whenever
     response arrives or error occurs.
@@ -318,15 +346,25 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBindTable, cbCtx):
         lookupMib, future = cbCtx
         if future.cancelled():
             return
-        future.set_result(
-            (errorIndication, errorStatus, errorIndex, [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in varBindTable])
-        )
+        try:
+            varBindsUnmade = [vbProcessor.unmakeVarBinds(snmpEngine,
+                                                         varBindTableRow,
+                                                         lookupMib)
+                              for varBindTableRow in varBindTable]
+        except Exception:
+            ex = sys.exc_info()[1]
+            future.set_exception(ex)
+        else:
+            future.set_result(
+                (errorIndication, errorStatus, errorIndex, varBindsUnmade)
+            )
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
 
@@ -340,12 +378,13 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     )
     return future
 
+
 @asyncio.coroutine
 def bulkCmd(snmpEngine, authData, transportTarget, contextData,
             nonRepeaters, maxRepetitions, *varBinds, **options):
     """Creates a generator to perform SNMP GETBULK query.
 
-    When itereator gets advanced by :py:mod:`asyncio` main loop,
+    When iterator gets advanced by :py:mod:`asyncio` main loop,
     SNMP GETBULK request is send (:RFC:`1905#section-4.2.3`).
     The iterator yields :py:class:`asyncio.Future` which gets done whenever
     response arrives or error occurs.
@@ -394,13 +433,31 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
         True value indicates SNMP PDU error.
     errorIndex : int
         Non-zero value refers to `varBinds[errorIndex-1]`
-    varBinds : tuple
+    varBindTable : tuple
         A sequence of sequences (e.g. 2-D array) of
         :py:class:`~pysnmp.smi.rfc1902.ObjectType` class instances
-        representing a table of MIB variables returned in SNMP response.
-        Inner sequences represent table rows and ordered exactly the same
-        as `varBinds` in request. Response to GETNEXT always contain
-        a single row.
+        representing a table of MIB variables returned in SNMP response, with
+        up to ``maxRepetitions`` rows, i.e.
+        ``len(varBindTable) <= maxRepetitions``.
+
+        For ``0 <= i < len(varBindTable)`` and ``0 <= j < len(varBinds)``,
+        ``varBindTable[i][j]`` represents:
+
+        - For non-repeaters (``j < nonRepeaters``), the first lexicographic
+          successor of ``varBinds[j]``, regardless the value of ``i``, or an
+          :py:class:`~pysnmp.smi.rfc1902.ObjectType` instance with the
+          :py:obj:`~pysnmp.proto.rfc1905.endOfMibView` value if no such
+          successor exists;
+        - For repeaters (``j >= nonRepeaters``), the ``i``-th lexicographic
+          successor of ``varBinds[j]``, or an
+          :py:class:`~pysnmp.smi.rfc1902.ObjectType` instance with the
+          :py:obj:`~pysnmp.proto.rfc1905.endOfMibView` value if no such
+          successor exists.
+
+        See :rfc:`3416#section-4.2.3` for details on the underlying
+        ``GetBulkRequest-PDU`` and the associated ``GetResponse-PDU``, such as
+        specific conditions under which the server may truncate the response,
+        causing ``varBindTable`` to have less than ``maxRepetitions`` rows.
 
     Raises
     ------
@@ -415,7 +472,7 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
     >>> @asyncio.coroutine
     ... def run():
-    ...     errorIndication, errorStatus, errorIndex, varBinds = yield from nextCmd(
+    ...     errorIndication, errorStatus, errorIndex, varBinds = yield from bulkCmd(
     ...         SnmpEngine(),
     ...         CommunityData('public'),
     ...         UdpTransportTarget(('demo.snmplabs.com', 161)),
@@ -426,19 +483,29 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     ...     print(errorIndication, errorStatus, errorIndex, varBinds)
     >>>
     >>> asyncio.get_event_loop().run_until_complete(run())
-    (None, 0, 0, [[ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.1.0')), DisplayString('SunOS zeus.snmplabs.com 4.1.3_U1 1 sun4m')), ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.2.0')), ObjectIdentifier('1.3.6.1.4.1.424242.1.1')]])
+    (None, 0, 0, [[ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.1.0')), DisplayString('SunOS zeus.snmplabs.com 4.1.3_U1 1 sun4m'))], [ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.2.0')), ObjectIdentifier('1.3.6.1.4.1.424242.1.1'))]])
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBindTable, cbCtx):
         lookupMib, future = cbCtx
         if future.cancelled():
             return
-        future.set_result(
-            (errorIndication, errorStatus, errorIndex, [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in varBindTable])
-        )
+        try:
+            varBindsUnmade = [vbProcessor.unmakeVarBinds(snmpEngine,
+                                                         varBindTableRow,
+                                                         lookupMib)
+                              for varBindTableRow in varBindTable]
+        except Exception:
+            ex = sys.exc_info()[1]
+            future.set_exception(ex)
+        else:
+            future.set_result(
+                (errorIndication, errorStatus, errorIndex, varBindsUnmade)
+            )
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
 
