@@ -1,16 +1,18 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
-# License: http://pysnmp.sf.net/license.html
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
 #
-import socket, sys
+import socket
+import sys
 from pysnmp.carrier.asyncio.dgram import udp, udp6
 from pysnmp.hlapi.transport import AbstractTransportTarget
 from pysnmp.error import PySnmpError
 
 __all__ = ['Udp6TransportTarget', 'UdpTransportTarget']
 
+
 class UdpTransportTarget(AbstractTransportTarget):
     """Creates UDP/IPv4 configuration entry and initialize socket API if needed.
 
@@ -47,6 +49,7 @@ class UdpTransportTarget(AbstractTransportTarget):
     """
     transportDomain = udp.domainName
     protoTransport = udp.UdpAsyncioTransport
+
     def _resolveAddr(self, transportAddr):
         try:
             return socket.getaddrinfo(transportAddr[0],
@@ -55,7 +58,9 @@ class UdpTransportTarget(AbstractTransportTarget):
                                       socket.SOCK_DGRAM,
                                       socket.IPPROTO_UDP)[0][4][:2]
         except socket.gaierror:
-            raise PySnmpError('Bad IPv4/UDP transport address %s: %s' % ('@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+            raise PySnmpError('Bad IPv4/UDP transport address %s: %s' % (
+                '@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+
 
 class Udp6TransportTarget(AbstractTransportTarget):
     """Creates UDP/IPv6 configuration entry and initialize socket API if needed.
@@ -105,6 +110,7 @@ class Udp6TransportTarget(AbstractTransportTarget):
     """
     transportDomain = udp6.domainName
     protoTransport = udp6.Udp6AsyncioTransport
+
     def _resolveAddr(self, transportAddr):
         try:
             return socket.getaddrinfo(transportAddr[0],
@@ -113,4 +119,5 @@ class Udp6TransportTarget(AbstractTransportTarget):
                                       socket.SOCK_DGRAM,
                                       socket.IPPROTO_UDP)[0][4][:2]
         except socket.gaierror:
-            raise PySnmpError('Bad IPv6/UDP transport address %s: %s' % ('@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+            raise PySnmpError('Bad IPv6/UDP transport address %s: %s' % (
+                '@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
