@@ -1,8 +1,8 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
-# License: http://pysnmp.sf.net/license.html
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
 #
 from pysnmp.entity.rfc3413 import cmdgen
 from pysnmp.smi.rfc1902 import *
@@ -19,6 +19,7 @@ lcd = CommandGeneratorLcdConfigurator()
 
 isEndOfMib = lambda x: not cmdgen.getNextVarBinds(x)[1]
 
+
 def getCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Performs SNMP GET query.
@@ -108,6 +109,7 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBinds, cbCtx):
@@ -129,6 +131,7 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
          options.get('cbFun'), options.get('cbCtx'))
     )
 
+
 def setCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Performs SNMP SET query.
@@ -218,6 +221,7 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBinds, cbCtx):
@@ -237,6 +241,7 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
                   options.get('cbFun'), options.get('cbCtx'))
     )
 
+
 def nextCmd(snmpEngine, authData, transportTarget, contextData,
             *varBinds, **options):
     """Performs SNMP GETNEXT query.
@@ -328,12 +333,14 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle, errorIndication,
                 errorStatus, errorIndex, varBindTable, cbCtx):
         lookupMib, cbFun, cbCtx = cbCtx
         return cbFun(snmpEngine, sendRequestHandle, errorIndication,
                      errorStatus, errorIndex,
-                     [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in varBindTable],
+                     [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in
+                      varBindTable],
                      cbCtx)
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
@@ -345,6 +352,7 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
                   options.get('cbFun'), options.get('cbCtx'))
     )
 
+
 def bulkCmd(snmpEngine, authData, transportTarget, contextData,
             nonRepeaters, maxRepetitions, *varBinds, **options):
     """Performs SNMP GETBULK query.
@@ -408,12 +416,30 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     * errorIndication (str): True value indicates SNMP engine error.
     * errorStatus (str): True value indicates SNMP PDU error.
     * errorIndex (int): Non-zero value refers to `varBinds[errorIndex-1]`
-    * varBinds (tuple): A sequence of sequences (e.g. 2-D array) of
-      :py:class:`~pysnmp.smi.rfc1902.ObjectType` class instances
-      representing a table of MIB variables returned in SNMP response.
-      Inner sequences represent table rows and ordered exactly the same
-      as `varBinds` in request. Number of rows might be less or equal
-      to `maxRepetitions` value in request.
+    * varBindTable (tuple):
+      A sequence of sequences (e.g. 2-D array) of
+      :py:class:`~pysnmp.smi.rfc1902.ObjectType` class instances representing a
+      table of MIB variables returned in SNMP response, with up to
+      ``maxRepetitions`` rows, i.e. ``len(varBindTable) <= maxRepetitions``.
+
+      For ``0 <= i < len(varBindTable)`` and ``0 <= j < len(varBinds)``,
+      ``varBindTable[i][j]`` represents:
+
+      - For non-repeaters (``j < nonRepeaters``), the first lexicographic
+        successor of ``varBinds[j]``, regardless the value of ``i``, or an
+        :py:class:`~pysnmp.smi.rfc1902.ObjectType` instance with the
+        :py:obj:`~pysnmp.proto.rfc1905.endOfMibView` value if no such successor
+        exists;
+      - For repeaters (``j >= nonRepeaters``), the ``i``-th lexicographic
+        successor of ``varBinds[j]``, or an
+        :py:class:`~pysnmp.smi.rfc1902.ObjectType` instance with the
+        :py:obj:`~pysnmp.proto.rfc1905.endOfMibView` value if no such successor
+        exists.
+
+      See :rfc:`3416#section-4.2.3` for details on the underlying
+      ``GetBulkRequest-PDU`` and the associated ``GetResponse-PDU``, such as
+      specific conditions under which the server may truncate the response,
+      causing ``varBindTable`` to have less than ``maxRepetitions`` rows.
     * `cbCtx` : Original user-supplied object.
 
     Returns
@@ -443,16 +469,19 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     ...         ObjectType(ObjectIdentity('SNMPv2-MIB', 'system')),
     ...         cbFun=cbFun)
     >>> snmpEngine.transportDispatcher.runDispatcher()
-    (None, 0, 0, [ [ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.1.0')), DisplayString('SunOS zeus.snmplabs.com 4.1.3_U1 1 sun4m')), ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.2.0')), ObjectIdentifier('1.3.6.1.4.1.424242.1.1')] ])
+    (None, 0, 0, [[ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.1.0')), DisplayString('SunOS zeus.snmplabs.com 4.1.3_U1 1 sun4m'))], [ObjectType(ObjectIdentity(ObjectName('1.3.6.1.2.1.1.2.0')), ObjectIdentifier('1.3.6.1.4.1.424242.1.1'))]])
     >>>
 
     """
+
     def __cbFun(snmpEngine, sendRequestHandle,
                 errorIndication, errorStatus, errorIndex,
                 varBindTable, cbCtx):
         lookupMib, cbFun, cbCtx = cbCtx
         return cbFun(snmpEngine, sendRequestHandle, errorIndication,
-                     errorStatus, errorIndex, [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in varBindTable], cbCtx)
+                     errorStatus, errorIndex,
+                     [vbProcessor.unmakeVarBinds(snmpEngine, varBindTableRow, lookupMib) for varBindTableRow in
+                      varBindTable], cbCtx)
 
     addrName, paramsName = lcd.configure(snmpEngine, authData, transportTarget)
     return cmdgen.BulkCommandGenerator().sendVarBinds(
