@@ -1,8 +1,8 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
-# License: http://pysnmp.sf.net/license.html
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
 #
 from sys import version_info
 from pysnmp.hlapi.asyncore import cmdgen
@@ -16,9 +16,11 @@ __all__ = ['getCmd', 'nextCmd', 'setCmd', 'bulkCmd']
 if version_info[:2] < (2, 6):
     __all__.append('next')
 
+    # noinspection PyShadowingBuiltins
     def next(iter):
         return iter.next()
 
+
 def getCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Creates a generator to perform one or more SNMP GET queries.
@@ -78,7 +80,7 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
 
     Examples
     --------
-    >>> from pysnmp.hlapi.asyncore import *
+    >>> from pysnmp.hlapi import *
     >>> g = getCmd(SnmpEngine(),
     ...            CommunityData('public'),
     ...            UdpTransportTarget(('demo.snmplabs.com', 161)),
@@ -89,6 +91,8 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
+    # noinspection PyShadowingNames
     def cbFun(snmpEngine, sendRequestHandle,
               errorIndication, errorStatus, errorIndex,
               varBinds, cbCtx):
@@ -121,6 +125,7 @@ def getCmd(snmpEngine, authData, transportTarget, contextData,
         if not varBinds:
             break
 
+
 def setCmd(snmpEngine, authData, transportTarget, contextData,
            *varBinds, **options):
     """Creates a generator to perform one or more SNMP SET queries.
@@ -181,7 +186,7 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
 
     Examples
     --------
-    >>> from pysnmp.hlapi.asyncore import *
+    >>> from pysnmp.hlapi import *
     >>> g = setCmd(SnmpEngine(),
     ...            CommunityData('public'),
     ...            UdpTransportTarget(('demo.snmplabs.com', 161)),
@@ -192,6 +197,8 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
     >>>
 
     """
+
+    # noinspection PyShadowingNames
     def cbFun(snmpEngine, sendRequestHandle,
               errorIndication, errorStatus, errorIndex,
               varBinds, cbCtx):
@@ -224,6 +231,7 @@ def setCmd(snmpEngine, authData, transportTarget, contextData,
         if not varBinds:
             break
 
+
 def nextCmd(snmpEngine, authData, transportTarget, contextData,
             *varBinds, **options):
     """Creates a generator to perform one or more SNMP GETNEXT queries.
@@ -258,12 +266,14 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
             * `lookupMib` - load MIB and resolve response MIB variables at
               the cost of slightly reduced performance. Default is `True`.
               Default is `True`.
-            * `lexicographicMode` - stop iteration when all response MIB
+            * `lexicographicMode` - walk SNMP agent's MIB till the end (if `True`),
+              otherwise (if `False`) stop iteration when all response MIB
               variables leave the scope of initial MIB variables in
               `varBinds`. Default is `True`.
             * `ignoreNonIncreasingOid` - continue iteration even if response
               MIB variables (OIDs) are not greater then request MIB variables.
-              Default is `False`.
+              Be aware that setting it to `True` may cause infinite loop between
+              SNMP management and agent applications. Default is `False`.
             * `maxRows` - stop iteration once this generator instance processed
               `maxRows` of SNMP conceptual table. Default is `0` (no limit).
             * `maxCalls` - stop iteration once this generator instance processed
@@ -297,7 +307,8 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     * SNMP :py:class:`~pysnmp.proto.rfc1905.EndOfMibView` values
       (also known as *SNMP exception values*) are reported for all
       MIB variables in `varBinds`
-    * *lexicographicMode* option is set to `False` and all
+    * *lexicographicMode* option is `True` and SNMP agent reports
+      end-of-mib or *lexicographicMode* is `False` and all
       response MIB variables leave the scope of `varBinds`
 
     At any moment a new sequence of `varBinds` could be send back into
@@ -305,7 +316,7 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
 
     Examples
     --------
-    >>> from pysnmp.hlapi.asyncore import *
+    >>> from pysnmp.hlapi import *
     >>> g = nextCmd(SnmpEngine(),
     ...             CommunityData('public'),
     ...             UdpTransportTarget(('demo.snmplabs.com', 161)),
@@ -316,6 +327,8 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     >>> g.send( [ ObjectType(ObjectIdentity('IF-MIB', 'ifInOctets')) ] )
     (None, 0, 0, [(ObjectName('1.3.6.1.2.1.2.2.1.10.1'), Counter32(284817787))])
     """
+
+    # noinspection PyShadowingNames
     def cbFun(snmpEngine, sendRequestHandle,
               errorIndication, errorStatus, errorIndex,
               varBindTable, cbCtx):
@@ -338,9 +351,11 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
     totalRows = totalCalls = 0
 
     while True:
+        previousVarBinds = varBinds
+
         if varBinds:
             cmdgen.nextCmd(snmpEngine, authData, transportTarget, contextData,
-                           *[(x[0], Null()) for x in varBinds],
+                           *[(x[0], Null('')) for x in varBinds],
                            **dict(cbFun=cbFun, cbCtx=cbCtx,
                                   lookupMib=options.get('lookupMib', True)))
 
@@ -350,12 +365,11 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
             errorStatus = cbCtx['errorStatus']
             errorIndex = cbCtx['errorIndex']
 
-            if ignoreNonIncreasingOid and errorIndication and \
-                   isinstance(errorIndication, errind.OidNotIncreasing):
+            if ignoreNonIncreasingOid and errorIndication and isinstance(errorIndication, errind.OidNotIncreasing):
                 errorIndication = None
 
             if errorIndication:
-                yield errorIndication, errorStatus, errorIndex, varBinds
+                yield (errorIndication, errorStatus, errorIndex, varBinds)
                 return
             elif errorStatus:
                 if errorStatus == 2:
@@ -363,16 +377,25 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
                     # from SNMPv1 Agent through internal pysnmp proxy.
                     errorStatus = errorStatus.clone(0)
                     errorIndex = errorIndex.clone(0)
-                yield errorIndication, errorStatus, errorIndex, varBinds
+                yield (errorIndication, errorStatus, errorIndex, varBinds)
                 return
             else:
+                stopFlag = True
+
                 varBinds = cbCtx['varBindTable'] and cbCtx['varBindTable'][0]
-                for idx, varBind in enumerate(varBinds):
+
+                for col, varBind in enumerate(varBinds):
                     name, val = varBind
-                    if not isinstance(val, Null):
-                        if lexicographicMode or initialVars[idx].isPrefixOf(name):
-                            break
-                else:
+                    if isinstance(val, Null):
+                        varBinds[col] = previousVarBinds[col][0], endOfMibView
+
+                    if not lexicographicMode and not initialVars[col].isPrefixOf(name):
+                        varBinds[col] = previousVarBinds[col][0], endOfMibView
+
+                    if stopFlag and varBinds[col][1] is not endOfMibView:
+                        stopFlag = False
+
+                if stopFlag:
                     return
 
                 totalRows += 1
@@ -381,17 +404,19 @@ def nextCmd(snmpEngine, authData, transportTarget, contextData,
             errorIndication = errorStatus = errorIndex = None
             varBinds = []
 
-        initialVarBinds = (yield errorIndication, errorStatus,\
-                                 errorIndex, varBinds)
+        initialVarBinds = (yield errorIndication, errorStatus, errorIndex, varBinds)
 
         if initialVarBinds:
             varBinds = initialVarBinds
             initialVars = [x[0] for x in vbProcessor.makeVarBinds(snmpEngine, varBinds)]
 
-        if maxRows and totalRows >= maxRows or \
-                 maxCalls and totalCalls >= maxCalls:
+        if maxRows and totalRows >= maxRows:
             return
 
+        if maxCalls and totalCalls >= maxCalls:
+            return
+
+
 def bulkCmd(snmpEngine, authData, transportTarget, contextData,
             nonRepeaters, maxRepetitions, *varBinds, **options):
     """Creates a generator to perform one or more SNMP GETBULK queries.
@@ -436,12 +461,14 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
             * `lookupMib` - load MIB and resolve response MIB variables at
               the cost of slightly reduced performance. Default is `True`.
               Default is `True`.
-            * `lexicographicMode` - stop iteration when all response MIB
+            * `lexicographicMode` - walk SNMP agent's MIB till the end (if `True`),
+              otherwise (if `False`) stop iteration when all response MIB
               variables leave the scope of initial MIB variables in
               `varBinds`. Default is `True`.
             * `ignoreNonIncreasingOid` - continue iteration even if response
               MIB variables (OIDs) are not greater then request MIB variables.
-              Default is `False`.
+              Be aware that setting it to `True` may cause infinite loop between
+              SNMP management and agent applications. Default is `False`.
             * `maxRows` - stop iteration once this generator instance processed
               `maxRows` of SNMP conceptual table. Default is `0` (no limit).
             * `maxCalls` - stop iteration once this generator instance processed
@@ -475,7 +502,8 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     * SNMP :py:class:`~pysnmp.proto.rfc1905.EndOfMibView` values
       (also known as *SNMP exception values*) are reported for all
       MIB variables in `varBinds`
-    * *lexicographicMode* option is set to `False` and all
+    * *lexicographicMode* option is `True` and SNMP agent reports
+      end-of-mib or *lexicographicMode* is `False` and all
       response MIB variables leave the scope of `varBinds`
 
     At any moment a new sequence of `varBinds` could be send back into
@@ -487,7 +515,7 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
 
     Examples
     --------
-    >>> from pysnmp.hlapi.asyncore import *
+    >>> from pysnmp.hlapi import *
     >>> g = bulkCmd(SnmpEngine(),
     ...             CommunityData('public'),
     ...             UdpTransportTarget(('demo.snmplabs.com', 161)),
@@ -499,6 +527,8 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
     >>> g.send( [ ObjectType(ObjectIdentity('IF-MIB', 'ifInOctets')) ] )
     (None, 0, 0, [(ObjectName('1.3.6.1.2.1.2.2.1.10.1'), Counter32(284817787))])
     """
+
+    # noinspection PyShadowingNames
     def cbFun(snmpEngine, sendRequestHandle,
               errorIndication, errorStatus, errorIndex,
               varBindTable, cbCtx):
@@ -524,11 +554,13 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
 
     while not stopFlag:
         if maxRows and totalRows < maxRows:
-            maxRepetitions = min(maxRepetitions, maxRows-totalRows)
+            maxRepetitions = min(maxRepetitions, maxRows - totalRows)
+
+        previousVarBinds = varBinds
 
         cmdgen.bulkCmd(snmpEngine, authData, transportTarget, contextData,
                        nonRepeaters, maxRepetitions,
-                       *[(x[0], Null()) for x in varBinds],
+                       *[(x[0], Null('')) for x in varBinds],
                        **dict(cbFun=cbFun, cbCtx=cbCtx,
                               lookupMib=options.get('lookupMib', True)))
 
@@ -544,8 +576,8 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
             errorIndication = None
 
         if errorIndication:
-            yield errorIndication, errorStatus, errorIndex, \
-                    varBindTable and varBindTable[0] or []
+            yield (errorIndication, errorStatus, errorIndex,
+                   varBindTable and varBindTable[0] or [])
             if errorIndication != errind.requestTimedOut:
                 return
         elif errorStatus:
@@ -554,29 +586,30 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
                 # from SNMPv1 Agent through internal pysnmp proxy.
                 errorStatus = errorStatus.clone(0)
                 errorIndex = errorIndex.clone(0)
-            yield errorIndication, errorStatus, errorIndex, \
-                    varBindTable and varBindTable[0] or []
+            yield (errorIndication, errorStatus, errorIndex, varBindTable and varBindTable[0] or [])
             return
         else:
-            for i in range(len(varBindTable)):
+            for row in range(len(varBindTable)):
                 stopFlag = True
-                if len(varBindTable[i]) != len(initialVars):
-                    varBindTable = i and varBindTable[:i-1] or []
+                if len(varBindTable[row]) != len(initialVars):
+                    varBindTable = row and varBindTable[:row - 1] or []
                     break
-                for j in range(len(varBindTable[i])):
-                    name, val = varBindTable[i][j]
-                    if nullVarBinds[j]:
-                        varBindTable[i][j] = name, endOfMibView
+                for col in range(len(varBindTable[row])):
+                    name, val = varBindTable[row][col]
+                    if row:
+                        previousVarBinds = varBindTable[row - 1]
+                    if nullVarBinds[col]:
+                        varBindTable[row][col] = previousVarBinds[col][0], endOfMibView
                         continue
                     stopFlag = False
                     if isinstance(val, Null):
-                        nullVarBinds[j] = True
-                    elif not lexicographicMode and \
-                                not initialVars[j].isPrefixOf(name):
-                        varBindTable[i][j] = name, endOfMibView
-                        nullVarBinds[j] = True
+                        varBindTable[row][col] = previousVarBinds[col][0], endOfMibView
+                        nullVarBinds[col] = True
+                    if not lexicographicMode and not initialVars[col].isPrefixOf(name):
+                        varBindTable[row][col] = previousVarBinds[col][0], endOfMibView
+                        nullVarBinds[col] = True
                 if stopFlag:
-                    varBindTable = i and varBindTable[:i-1] or []
+                    varBindTable = row and varBindTable[:row - 1] or []
                     break
 
             totalRows += len(varBindTable)
@@ -584,16 +617,19 @@ def bulkCmd(snmpEngine, authData, transportTarget, contextData,
 
             if maxRows and totalRows >= maxRows:
                 if totalRows > maxRows:
-                    varBindTable = varBindTable[:-(totalRows-maxRows)]
+                    varBindTable = varBindTable[:-(totalRows - maxRows)]
                 stopFlag = True
 
             if maxCalls and totalCalls >= maxCalls:
                 stopFlag = True
 
-            for varBinds in varBindTable:
-                initialVarBinds = (yield errorIndication, errorStatus,\
-                                         errorIndex, varBinds)
+            varBinds = varBindTable and varBindTable[-1] or []
+
+            for varBindRow in varBindTable:
+                initialVarBinds = (yield errorIndication, errorStatus, errorIndex, varBindRow)
 
                 if initialVarBinds:
                     varBinds = initialVarBinds
                     initialVars = [x[0] for x in vbProcessor.makeVarBinds(snmpEngine, varBinds)]
+                    nullVarBinds = [False] * len(initialVars)
+
