@@ -1,16 +1,18 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
-# License: http://pysnmp.sf.net/license.html
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
 #
-import socket, sys
+import socket
+import sys
 from pysnmp.carrier.asyncore.dgram import udp, udp6, unix
 from pysnmp.hlapi.transport import AbstractTransportTarget
 from pysnmp import error
 
 __all__ = ['UnixTransportTarget', 'Udp6TransportTarget', 'UdpTransportTarget']
 
+
 class UdpTransportTarget(AbstractTransportTarget):
     """Creates UDP/IPv4 configuration entry and initialize socket API if needed.
 
@@ -22,20 +24,21 @@ class UdpTransportTarget(AbstractTransportTarget):
 
     Parameters
     ----------
-    transportAddr : tuple
+    transportAddr: :py:class:`tuple`
         Indicates remote address in Python :py:mod:`socket` module format
         which is a tuple of FQDN, port where FQDN is a string representing
         either hostname or IPv4 address in quad-dotted form, port is an
         integer.
-    timeout : int
+    timeout: :py:class:`int`
         Response timeout in seconds.
-    retries : int
+    retries: :py:class:`int`
         Maximum number of request retries, 0 retries means just a single
         request.
-    tagList : str
-        Arbitrary string that contains a list of tag values which are used
-        to select target addresses for a particular operation
-        (:RFC:`3413#section-4.1.4`).
+    tagList: :py:class:`str`
+        Arbitrary string that contains a list of space-separated tag
+        strings used to select target addresses and/or SNMP configuration
+        (see :RFC:`3413#section-4.1.1`, :RFC:`2576#section-5.3` and
+        :py:class:`~pysnmp.hlapi.CommunityData` object).
 
     Examples
     --------
@@ -47,6 +50,7 @@ class UdpTransportTarget(AbstractTransportTarget):
     """
     transportDomain = udp.domainName
     protoTransport = udp.UdpSocketTransport
+
     def _resolveAddr(self, transportAddr):
         try:
             return socket.getaddrinfo(transportAddr[0],
@@ -55,7 +59,9 @@ class UdpTransportTarget(AbstractTransportTarget):
                                       socket.SOCK_DGRAM,
                                       socket.IPPROTO_UDP)[0][4][:2]
         except socket.gaierror:
-            raise error.PySnmpError('Bad IPv4/UDP transport address %s: %s' % ('@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+            raise error.PySnmpError('Bad IPv4/UDP transport address %s: %s' % (
+                '@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+
 
 class Udp6TransportTarget(AbstractTransportTarget):
     """Creates UDP/IPv6 configuration entry and initialize socket API if needed.
@@ -102,6 +108,7 @@ class Udp6TransportTarget(AbstractTransportTarget):
     """
     transportDomain = udp6.domainName
     protoTransport = udp6.Udp6SocketTransport
+
     def _resolveAddr(self, transportAddr):
         try:
             return socket.getaddrinfo(transportAddr[0],
@@ -110,7 +117,9 @@ class Udp6TransportTarget(AbstractTransportTarget):
                                       socket.SOCK_DGRAM,
                                       socket.IPPROTO_UDP)[0][4][:2]
         except socket.gaierror:
-            raise error.PySnmpError('Bad IPv6/UDP transport address %s: %s' % ('@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+            raise error.PySnmpError('Bad IPv6/UDP transport address %s: %s' % (
+                '@'.join([str(x) for x in transportAddr]), sys.exc_info()[1]))
+
 
 class UnixTransportTarget(AbstractTransportTarget):
     transportDomain = unix.domainName
