@@ -1,8 +1,16 @@
+#
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
+#
 from pysnmp.proto.secmod import cache
 from pysnmp.proto import error
 
-class AbstractSecurityModel:
+
+class AbstractSecurityModel(object):
     securityModelID = None
+
     def __init__(self):
         self._cache = cache.Cache()
 
