@@ -1,3 +1,9 @@
+#
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysnmp/license.html
+#
 MibScalarInstance, = mibBuilder.importSymbols(
     'SNMPv2-SMI',
     'MibScalarInstance'
@@ -13,7 +19,8 @@ MibScalarInstance, = mibBuilder.importSymbols(
 )
 
 __snmpTargetSpinLock = MibScalarInstance(snmpTargetSpinLock.name, (0,), snmpTargetSpinLock.syntax.clone(0))
-__snmpUnavailableContexts = MibScalarInstance(snmpUnavailableContexts.name, (0,), snmpUnavailableContexts.syntax.clone(0))
+__snmpUnavailableContexts = MibScalarInstance(snmpUnavailableContexts.name, (0,),
+                                              snmpUnavailableContexts.syntax.clone(0))
 __snmpUnknownContexts = MibScalarInstance(snmpUnknownContexts.name, (0,), snmpUnknownContexts.syntax.clone(0))
 
 mibBuilder.exportSymbols(
@@ -22,4 +29,3 @@ mibBuilder.exportSymbols(
     snmpUnavailableContexts=__snmpUnavailableContexts,
     snmpUnknownContexts=__snmpUnknownContexts
 )
-
